/******************************************************************************
 * GroupManager.java - Manage the Groups of Players
 * $Id: GroupManager.java,v 1.5 2011/01/01 20:47:07 dick Exp $
 * 
 * BuckoFIBS - Backgammon by BuckoSoft
 * Copyright© 2010 - Dick Balaska - BuckoSoft, Corp.
 * 
 * $Log: GroupManager.java,v $
 * Revision 1.5  2011/01/01 20:47:07  dick
 * Add edit group.  Add getGroupsSelectedString for the groups text box.
 *
 * Revision 1.4  2011/01/01 06:16:44  dick
 * Filtering of players is functional.
 *
 * Revision 1.3  2011/01/01 02:33:11  dick
 * Database for GroupOfPlayers is fully functional.
 *
 * Revision 1.2  2011/01/01 00:19:29  dick
 * Lotso work.
 *
 * Revision 1.1  2010/12/31 05:28:38  dick
 * Manage the Groups of Players
 *
 */

/* 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * The Original Code is BuckoFIBS, <http://www.buckosoft.com/BuckoFIBS/>.
 * The Initial Developer of the Original Code is Dick Balaska and BuckoSoft, Corp.
 * 
 */
package com.buckosoft.fibs.BuckoFIBS;

import java.awt.Color;
import java.util.LinkedList;
import java.util.List;

import com.buckosoft.fibs.BuckoFIBS.db.Database;
import com.buckosoft.fibs.BuckoFIBS.gui.playerList.PlayerListPane;
import com.buckosoft.fibs.BuckoFIBS.gui.playerList.group.GroupPopupSubmenu;
import com.buckosoft.fibs.BuckoFIBS.gui.playerList.group.GroupSelectPane;
import com.buckosoft.fibs.BuckoFIBS.gui.playerList.group.GroupPopupSubmenu.PopupGroup;
import com.buckosoft.fibs.domain.GroupOfPlayers;
import com.buckosoft.fibs.domain.Player;
import com.buckosoft.fibs.domain.PlayerGroup;

/** Manage the GroupOfPlayers.
 * @author Dick Balaska
 * @since 2010/12/30
 * @version $Revision: 1.5 $ <br> $Date: 2011/01/01 20:47:07 $
 * @see <a href="http://cvs.buckosoft.com/Projects/BuckoFIBS/BuckoFIBS/src/main/java/com/buckosoft/fibs/BuckoFIBS/GroupManager.java">cvs GroupManager.java</a>
 */
public class GroupManager {
	private	final static boolean DEBUG = false;
	//protected final Log logger = LogFactory.getLog(getClass());	// XXX: I can't get log4j this working... 
	
	private List<GroupOfPlayers> groupList = new LinkedList<GroupOfPlayers>();
	private GroupSelectPane groupSelectPane;
	private	PlayerListPane	playerListPane;
	private	boolean	allGroupSelected;			// convienence
	
	private	Database	database;
	
	public void initDefaultGroups() {
		groupList = new LinkedList<GroupOfPlayers>();
		GroupOfPlayers gop;
		gop = new GroupOfPlayers("All", Color.black);
		gop.setActive(true);
		allGroupSelected = true;
		groupList.add(gop);
		groupList.add(new GroupOfPlayers("Friends", Color.blue));
		gop = new GroupOfPlayers("Enemies", Color.red);
		gop.setIgnore(true);
		groupList.add(gop);
	}
	
	public GroupManager() {
		groupList.add(new GroupOfPlayers("All", Color.black));
	}

	public void setGroupSelectPane(GroupSelectPane groupSelectPane) {
		this.groupSelectPane = groupSelectPane;
	}

	public void setPlayerListPane(PlayerListPane playerListPane) {
		this.playerListPane = playerListPane;
	}

	/** Set the reference to the database.
	 * @param database the database to set
	 */
	public void setDatabase(Database database) {
		this.database = database;
		groupList = null;
		try {
			groupList = this.database.getGroupsOfPlayers();
		} catch (Exception e) {}
		if (groupList == null || groupList.size() == 0)
			initDefaultGroups();
		this.allGroupSelected = groupList.get(0).isActive();
	}
	
	public void save() {
		if (DEBUG)
			System.out.println("GroupManager: save");
		this.database.store(groupList);
	}

	public void populatePane() {
		for (GroupOfPlayers gop : groupList) {
			groupSelectPane.addGroup(gop);
		}
		groupSelectPane.populateFinish();
	}
	
	/** Tag the active status for this GroupOfPlayers
	 * @param id
	 * @param groupName
	 * @param active
	 */
	public void updateGroupOfPlayers(int id, String groupName, boolean active) {
		if (DEBUG)
			System.out.println("GroupManager: updateGroupOfPlayers: id=" + id + " name=" + groupName + " active=" + active);
		if (id == 0) {
			GroupOfPlayers gop = new GroupOfPlayers(groupName, Color.black);
			gop.setActive(active);
			groupList.add(gop);
		} else {
			for (GroupOfPlayers gop : groupList) {
				if (gop.getId() == id) {
					if (active != gop.isActive())
						gop.setDirty(true);
					gop.setActive(active);
				}
			}
		}
		this.allGroupSelected = groupList.get(0).isActive();
		this.playerListPane.groupPaneChanged();
	}

	/** Get the GroupOfPlayers with this name
	 * @param groupName The name to search for
	 * @return The GroupOfPlayers or null if not found.
	 */
	public GroupOfPlayers getGroupOfPlayers(String groupName) {
		for (GroupOfPlayers gop : groupList) {
			if (gop.getGroupName().equals(groupName))
				return(gop);
		}
		return(null);
	}

	public GroupOfPlayers getGroupOfPlayers(int groupId) {
		for (GroupOfPlayers gop : groupList) {
			if (gop.getId() == groupId)
				return(gop);
		}
		return(null);
		
	}
	/** The user has clicked OK in the add new group dialog.
	 * @param groupOfPlayers The new GroupOfPlayers created by the user.
	 */
	public void addNewGroupFromUser(GroupOfPlayers groupOfPlayers) {
		groupList.add(groupOfPlayers);
		save();
		groupSelectPane.repopulate();
		populatePane();
		this.playerListPane.groupPaneChanged();
	}
	
	public void editGroupFromUser(GroupOfPlayers groupOfPlayers) {
		GroupOfPlayers gop = getGroupOfPlayers(groupOfPlayers.getId());
		if (gop == null)
			return;
		gop.setDirty(true);
		save();
		groupSelectPane.repopulate();
		populatePane();
		this.playerListPane.groupPaneChanged();
	}

	/** Handle an event from the popup menu
	 * @param groupName
	 * @param playerName
	 */
	public void popupMenuItemSelected(String groupName, String playerName) {
		Player p = this.database.getPlayer(playerName);
		if (p == null) {
			p = this.playerListPane.getPlayer(playerName);
			if (p == null)
				return;					// We can't find the guy we clicked on??
			this.database.store(p);		// Write him out so we get a playerId
			//System.out.println("New playerid =" + p.getId());
			//p = this.database.getPlayer(playerName);
			this.playerListPane.playerChanged(p);
			System.out.println("GroupManager: popupMenuItemSelected: name = '" + p.getName() + "' Now playerid =" + p.getId());
		}
		int pi = p.getId();
		GroupOfPlayers gop = getGroupOfPlayers(groupName);
		if (this.isInGroup(groupName, pi)) {
			for (PlayerGroup pg : gop.getPlayerGroups()) {
				if (pg.getPlayerId() == pi) {
					pg.setTagForDelete(true);
					gop.setDirty(true);
					this.save();
					return;
				}
			}
		} else {
			gop.addPlayer(pi);
			gop.setDirty(true);
			this.save();
		}
	}

	/** Is this playerId in this group?
	 * @param groupName The name of the group to check
	 * @param playerId The id of the player in question
	 * @return Success
	 */
	public boolean isInGroup(String groupName, int playerId) {
		GroupOfPlayers gop = getGroupOfPlayers(groupName);
		LinkedList<Integer> ll = gop.getPlayers();
		for (int i : ll) {
			if (i == playerId)
				return(true);
		}
		return(false);
	}

	public LinkedList<PopupGroup> getGroupsForPlayer(GroupPopupSubmenu groupPopupSubmenu, int playerId) {
		LinkedList<PopupGroup> ll = new LinkedList<PopupGroup>();
		for (GroupOfPlayers gop : groupList) {
			PopupGroup pg = groupPopupSubmenu.new PopupGroup();
			pg.name = gop.getGroupName();
			for (Integer i : gop.getPlayers()) {
				if (playerId == i) {
					pg.selected = true;
					break;
				}
			}
			ll.add(pg);
		}
		return(ll);
	}
	/** Should this player be filtered?
	 * @param player The Player to test
	 * @return true = ok to use this player
	 */
	public boolean filter(Player player) {
		if (this.allGroupSelected)
			return(true);
		if (player.getId() == 0)
			return(false);
		int playerId = player.getId();
		for (GroupOfPlayers gop : groupList) {
			if (gop.isActive()) {
				for (Integer i : gop.getPlayers()) {
					if (playerId == i) {
						return(!gop.isIgnore());
					}
				}
			}
		}		
		return(false);
	}
	
	/** Get the String that is displayed at the top of the PlayerListPane
	 * @return A nice String
	 */
	public String getGroupsSelectedString() {
		StringBuffer sb = new StringBuffer();
		if (this.allGroupSelected)
			sb.append("All");
		for (GroupOfPlayers gop : groupList) {
			if (gop.isIgnore()) {
				if (gop.isActive()) {
					if (sb.length() != 0)
						sb.append("; ");
					sb.append("!");
					sb.append(gop.getGroupName());
				}
			} else {
				if (this.allGroupSelected || !gop.isActive())
					continue;
				if (sb.length() != 0)
					sb.append("; ");
				sb.append(gop.getGroupName());
			}
		}
		return(sb.toString());
	}
}
