/******************************************************************************
 * HibernateUtil.java - Non-object access to the database 
 * $Id: HibernateUtil.java,v 1.3 2010/03/03 13:12:21 inim Exp $
 * 
 * BuckoFIBS - Backgammon by BuckoSoft
 * Copyright© 2009,2010 - Dick Balaska - BuckoSoft, Corp.
 * 
 * $Log: HibernateUtil.java,v $
 * Revision 1.3  2010/03/03 13:12:21  inim
 * Replaced (c) sign in comment mangled by CVS default encoding back to UTF-8
 *
 * Revision 1.2  2010/03/03 12:19:48  inim
 * Moved source to UTF8 encoding from CP1252 encoding. To this end all source files' (c) message was updated to "Copyright© 2009,2010 - Dick Balaska - BuckoSoft, Corp.". This replaces the (c) sign to UTF8, and adds the new year 2010.
 *
 * Revision 1.1  2010/02/04 05:57:53  inim
 * Mavenized project folder layout
 *
 * Revision 1.1  2009/02/20 10:24:51  dick
 * Non-object access to the database.
 *
 */

/* 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * The Original Code is BuckoFIBS, <http://www.buckosoft.com/BuckoFIBS/>.
 * The Initial Developer of the Original Code is Dick Balaska and BuckoSoft, Corp.
 */
package com.buckosoft.fibs.BuckoFIBS.db;

import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.cfg.Configuration;
import org.hibernate.tool.hbm2ddl.SchemaExport;

/** Non-object access to the database. <br>
 * This class is really private to DatabaseImpl.
 * @author Dick Balaska
 * @since 2009/02/18
 * @version $Revision: 1.3 $ <br> $Date: 2010/03/03 13:12:21 $
 * @see <a href="http://cvs.buckosoft.com/Projects/BuckoFIBS/BuckoFIBS/src/com/buckosoft/fibs/BuckoFIBS/db/HibernateUtil.java">cvs HibernateUtil.java</a>
 */
public class HibernateUtil {
    private static final SessionFactory sessionFactory;
    private	static Configuration configuration = null;
    static {
        try {
            // Create the SessionFactory from hibernate.cfg.xml
        	configuration = new Configuration().configure();
            sessionFactory = configuration.buildSessionFactory();
        } catch (Throwable ex) {
            // Make sure you log the exception, as it might be swallowed
            System.err.println("Initial SessionFactory creation failed." + ex);
            throw new ExceptionInInitializerError(ex);
        }
    }

    protected static SessionFactory getSessionFactory() {
        return sessionFactory;
    }

    protected static void maybeCreateNewDatabase() {
		Session session = getSessionFactory().getCurrentSession();
		session.beginTransaction();
		Object o = null;
		try {
			o= session.createQuery("from Player where id = ?")
					.setInteger(0, 1).uniqueResult();
		} catch (Exception e) {
			System.out.println("maybeCreateNewDatabase needs to create database");
			//e.printStackTrace();
		}
		session.getTransaction().commit();
    	if (o != null)		// database ok.
    		return;
    	
    	SchemaExport se = new SchemaExport(configuration);
    	se.execute(true, true, false, true);
    }
}
