/******************************************************************************
 * BoardGui.java - Main object of the GUI board.
 * $Id: BoardGui.java,v 1.4 2010/12/30 04:12:58 dick Exp $
 * 
 * BuckoFIBS - Backgammon by BuckoSoft
 * Copyright© 2009,2010 - Dick Balaska - BuckoSoft, Corp.
 * 
 * $Log: BoardGui.java,v $
 * Revision 1.4  2010/12/30 04:12:58  dick
 * Bug 18 -
 * If you Left Click first that will use the Left die,
 * if you Left Click again you should be able to then use the Right die.
 *
 * Revision 1.3  2010/03/03 13:12:21  inim
 * Replaced (c) sign in comment mangled by CVS default encoding back to UTF-8
 *
 * Revision 1.2  2010/03/03 12:19:49  inim
 * Moved source to UTF8 encoding from CP1252 encoding. To this end all source files' (c) message was updated to "Copyright© 2009,2010 - Dick Balaska - BuckoSoft, Corp.". This replaces the (c) sign to UTF8, and adds the new year 2010.
 *
 * Revision 1.1  2010/02/04 05:57:53  inim
 * Mavenized project folder layout
 *
 * Revision 1.8  2009/02/17 14:44:30  dick
 * Turn off DEBUG.
 *
 * Revision 1.7  2009/02/14 12:27:11  dick
 * Ignore double-clicks on the double cube.
 *
 * Revision 1.6  2009/02/12 06:27:48  dick
 * Check double-click before checkers-on-bar so we can move only one checker off the bar.
 *
 * Revision 1.5  2009/02/11 09:06:28  dick
 * Add youCantMove() to disable the hover helper after it got turned on at the beginning of your turn.
 *
 * Revision 1.4  2009/02/06 07:56:49  dick
 * Working on doubles.
 *
 * Revision 1.3  2009/02/03 09:47:08  dick
 * Use SEND_MOVE instead of SEND_COMMAND so we can make a "pick up the dice" sound.
 *
 * Revision 1.2  2009/01/31 08:49:44  dick
 * YourMove is initiated by GameManager and doesn't need the 2-shot setup anymore.
 *
 * Revision 1.1  2009/01/28 19:37:28  dick
 * package com.buckosoft.fibs.gui.board becomes com.buckosoft.fibs.BuckoFIBS.gui.board.
 *
 * Revision 1.8  2009/01/28 08:32:49  dick
 * Turn off DEBUG.
 *
 * Revision 1.7  2009/01/27 06:57:01  dick
 * Adhere to bearing off rules.
 *
 * Revision 1.6  2009/01/27 05:46:29  dick
 * Played first game against myself on Jibs!
 *
 * Revision 1.5  2009/01/26 07:23:43  dick
 * Working on moving checkers.
 *
 * Revision 1.4  2009/01/24 17:06:55  dick
 * Starting your turn is a 2 part operation.
 * First we get a "It's your turn" messages, then we get a board with the dice.
 *
 * Revision 1.3  2009/01/22 05:04:20  dick
 * More moving checkers on the board.  There are issues with playing 1-24 vs. 24-1.
 *
 * Revision 1.2  2009/01/20 07:42:26  dick
 * Move checkers on the board.
 *
 * Revision 1.1  2009/01/18 05:04:58  dick
 * Main object of the GUI board.
 */

/* 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * The Original Code is BuckoFIBS, <http://www.buckosoft.com/BuckoFIBS/>.
 * The Initial Developer of the Original Code is Dick Balaska and BuckoSoft, Corp.
 * 
 */
package com.buckosoft.fibs.BuckoFIBS.gui.board;

import java.awt.Point;

import com.buckosoft.fibs.BuckoFIBS.AudioManager;
import com.buckosoft.fibs.BuckoFIBS.CommandDispatcher;

/** Main object of the GUI board.  This object contains the controlling routines
 * for the user to manipulate the board.
 * It is a superclass of {@link BoardPane} so that we can get at its drawing methods. 
 * @author Dick Balaska
 * @since 2009/01/17
 * @version $Revision: 1.4 $ <br> $Date: 2010/12/30 04:12:58 $
 * @see <a href="http://cvs.buckosoft.com/Projects/BuckoFIBS/BuckoFIBS/src/com/buckosoft/fibs/BuckoFIBS/gui/board/BoardGui.java">cvs BoardGui.java</a>
 */
public class BoardGui extends BoardPane {
	private	final static boolean DEBUG = false;
	private	final static boolean DEBUG_POINT_POSITION = false;	// what point we are on
	
	private static final long serialVersionUID = 1L;

	private	CommandDispatcher commandDispatcher;  //  @jve:decl-index=0:

	private int	lastMousePosition = POINT_UNKNOWN;

	/** Are we in the middle of taking our turn? */
	private boolean boardActive = false;
	
	/** build up a string of our move to send to fibs */
	private	String	moveString;

	private int[]	myDice;
	
	/** Set the reference to the command dispatcher
	 * @param commandDispatcher Our system CommandDispatcher
	 */
	public	void	setCommandDispatcher(CommandDispatcher commandDispatcher) {
		this.commandDispatcher = commandDispatcher;
		this.properties = this.commandDispatcher.getProperties();
	}

	/** The constructor for the BoardGui
	 */
	BoardGui() {
		super();
		diceUsed[0] = false;
		diceUsed[1] = false;
	}

	/** The board has changed and needs to be redrawn and possibly manipulated
	 */
	public void updateBoard() {
		// this.repaint();		// handled in BoardTab.updateBoard();
	}

	/** Turn off this player's turn
	 */
	public void youCantMove() {
		this.boardActive = false;
	}

	/** Start your turn Part 1.  Set "our turn" and wait for the dice (a board command)
	 */
	public void yourMove(int diceToMove) {
		if (DEBUG)
			System.out.println("yourMove");
		boardActive = true;
		myDice = board.getDice()[0];
		if (myDice[0] == 0)
			return;
		moveString = "move";
		if (myDice[0] == myDice[1]) {
			if (diceUsed.length != 4)
				diceUsed = new boolean[4];
			int d = myDice[0];
			myDice = new int[4];
			for (int i=0; i<4; i++) {
				diceUsed[i] = false;
				myDice[i] = d;
			}
		} else {
			if (diceUsed.length != 2)
				diceUsed = new boolean[2];
			diceUsed[0] = false;
			diceUsed[1] = false;
		}
	}

	/** Handle the mouse being pressed.
	 * @param e The Event 
	 */
	protected void onMousePressed(java.awt.event.MouseEvent e) {
		if (!boardActive 
				&& !board.isYourTurnToRollOrDouble()
				&& !board.isAcceptDeclineDouble())
			return;
		if (DEBUG)
			System.out.println("press:" + e.getButton() + " x/y=" + e.getX() + "/" + e.getY());
		int p = getPointFromXY(e.getX(), e.getY());
		if (board.isYourTurnToRollOrDouble()) {
			if (e.getClickCount() == 2)
				return;
			if (p == POINT_BAR_DICE) {
				this.commandDispatcher.dispatch(CommandDispatcher.Command.SEND_ROLL);
				return;
			}
			if (p == POINT_DOUBLE) {
				this.commandDispatcher.dispatch(CommandDispatcher.Command.SEND_DOUBLE);
				return;
			}
			return;
		}
		if (board.isAcceptDeclineDouble()) {
			if (e.getClickCount() == 2)
				return;
			if (p == POINT_DOUBLE) {
				this.commandDispatcher.dispatch(CommandDispatcher.Command.SEND_ACCEPT);
				return;
			}
			if (p == POINT_REJECT) {
				this.commandDispatcher.dispatch(CommandDispatcher.Command.SEND_REJECT);
				return;
			}
			return;
		}
		if (e.getClickCount() == 2) {
			if (DEBUG)
				System.out.println("doubleclick:" + p);
			if (p == POINT_DICE) {
				this.boardActive = false;
				this.commandDispatcher.dispatch(CommandDispatcher.Command.SEND_MOVE, this.moveString);
				return;
			}
		}
		if (p == POINT_DICE && e.getButton() == 3) {
			int x = myDice[0];
			myDice[0] = myDice[1];
			myDice[1] = x;
			repaint();
			return;
		}
		if (board.getBar()[0] != 0 && p != POINT_BAR)	//must move off the bar first
			return;
		if (p > 0 || p == POINT_BAR) {
			int d = e.getButton() == 1 ? 0 : 1;
			if (myDice[0] == myDice[1]) {
				if (!diceUsed[3])
					d = 3;
				else if (!diceUsed[2])
					d = 2;
				else if (!diceUsed[1])
					d = 1;
				else if (!diceUsed[0])
					d = 0;
				else
					return;
			} else {
				if (diceUsed[d]) {
					d = d == 1 ? 0 : 1;
					if (diceUsed[d])
						return;
				}
			}
			moveChecker(p, d);
			hoverOverPoint(p);
			repaint();
			return;
		}
	}

	protected void onMouseMoved(java.awt.event.MouseEvent e) {
		if (!boardActive)
			return;
		int p = getPointFromXY(e.getX(), e.getY());
		if (lastMousePosition != p) {
			lastMousePosition = p;
			if (DEBUG_POINT_POSITION)
				System.out.println("p=" + p);
			onMouseReallyMoved(p);
		}
	}
	
	/** Mouse has entered a new point on the board
	 */
	private void onMouseReallyMoved(int p) {
		if (p > 0 || p == POINT_BAR)
			hoverOverPoint(p);
		else {
			setHoverPoint(-1);
			repaint();
		}
	}

	private void hoverOverPoint(int p) {
		int i;
		if (myDice == null)		// bogus hover detected
			return;
		// hover over bar?
		if (p == POINT_BAR) {
			if (board.getBar()[0] == 0) {
				setHoverPoint(-1);
				repaint();
				return;
			}
		} else if (board.getPoints()[p] >= 0) {
			if (hoverPoint != -1) {
				setHoverPoint(-1);
				repaint();
			}
			return;
		}
		for (i=0; i<diceUsed.length; i++) {
			if (!diceUsed[i])
				break;
		}
		if (i == diceUsed.length) {
			setHoverPoint(-1);
			repaint();
			return;
		}
		setHoverPoint(p);
		if (myDice[0] == myDice[1]) {
			for (i=diceUsed.length-1; i>=0; i--) {
				if (!diceUsed[i])
					checkDiceMove(p, myDice[0]);
			}
		} else {
			if (!diceUsed[0])
				checkDiceMove(p, myDice[0]);
			if (!diceUsed[1])
				checkDiceMove(p, myDice[1]);
		}
		repaint();
	}

	private void checkDiceMove(int p, int d) {
		if (isBearingOff()) {
			if (board.getDirection() == -1) {
				if (d >= p) {
					hoverTo[0] = 1;		// hover to home
					return;
				}
			} else {
				int r = 25-p;
				if (d >= r) {
					hoverTo[0] = 1;		// hover to home
					return;
				}
			}
		}
		if (p == POINT_BAR) {
			if (board.getDirection() == 1)
				p = d;
			else
				p = 25 - d;
			if (board.getPoints()[p] <= 1)
				hoverTo[p] = 1;
			return;
		}
		if (board.getDirection() == -1)
			p = p-d;
		else
			p = p+d;
		if (p > 24 || p < 1)
			return;
		if (board.getPoints()[p] <= 1)
			hoverTo[p] = 1;
	}

	private void moveChecker(int p, int di) {
		int p1;
		int d = myDice[di];
		if (p == POINT_BAR) {
			if (board.getDirection() == 1)
				p = d;
			else
				p = 25 - d;
			if (board.getBar()[0] > 0) {
				if (board.getPoints()[p] >= 2)
					return;
				if (board.getPoints()[p] == 1) {
					board.getBar()[1]++;
					board.getPoints()[p] = 0;
				}
				board.getPoints()[p]--;
				board.getBar()[0]--;
				diceUsed[di] = true;
				moveString += " bar-" + p;
			}
			return;
		}
		if (board.getDirection() == -1)
			p1 = p-d;
		else
			p1 = p+d;
		if (board.getPoints()[p] < 0) {
			if (isBearingOff()) {
				if (board.getDirection() == -1) {
					if (d >= p) {
						moveToHome(p, di);
						return;
					}
				} else {
					int r = 25-p;
					if (d >= r) {
						moveToHome(p, di);
						return;
					}
				}
			}
			if (board.getPoints()[p1] == 1) {
				board.getBar()[1]++;
				board.getPoints()[p1] = 0;
			}
			if (board.getPoints()[p1] >= 2) {
				this.commandDispatcher.dispatch(CommandDispatcher.Command.PLAY_CUE, AudioManager.Cue.CantMoveHere);
				return;
			}
			board.getPoints()[p1]--;
			board.getPoints()[p]++;
			diceUsed[di] = true;
			moveString += " " + p + "-" + p1;
		}
	}

	private void moveToHome(int p, int di) {
		board.getPoints()[p]++;
		board.getHome()[0]++;
		diceUsed[di] = true;
		moveString += " " + p + "-off";
	}

	private boolean isBearingOff() {
		int i;
		if (board.getDirection() == -1) {
			for (i=7; i<=24; i++)
				if (board.getPoints()[i] < 0)
					return(false);
		} else {
			for (i=1; i<=18; i++)
				if (board.getPoints()[i] < 0)
					return(false);
		}
		return(true);
	}

	private int getPointFromXY(int x, int y) {
		int p = POINT_UNKNOWN;
		int dx, dy;
		// Check where in the x axis did we click 
		// click in the double cube area?
		int w = getDoubleCubeWidth();
		Point t = getDoubleCubeXY();
		if (x >= t.x && x <= t.x+w
		 && y >= t.y && y <= t.y+w) {
			return(POINT_DOUBLE);
		}
		if (board.isAcceptDeclineDouble()) {
			dx = getDeclineX();
			dy = getDeclineY();
			w = bs.c15;
			if (x >= dx && x <= dx+w
			 && y >= dy && y <= dy+w) {
				return(POINT_REJECT);
			}
			return(POINT_UNKNOWN);
			
		}
		if (x < bs.w/2-bs.barWidth/2) {
			p = (x-bs.c15) / bs.checkerSize + 1;
			if (p == 7)		// XXX: why is this slop in here??
				p = 6;
		// click in the bar?
		} else if (x < bs.w/2-bs.barWidth/2 + bs.barWidth) {
			p = POINT_BAR;
			if (board.isYourTurnToRollOrDouble()) {
				if (y >= bs.h-bs.barWidth/2)
					p = POINT_BAR_DICE;
			}
		} else if (x < bs.w-bs.c15) {
			dy = bs.h/2-bs.c2;
			if (y > dy && y < dy+bs.dieSize) {
				dx = getPointX(4); 
				if (x > dx && x < dx + bs.c15 + bs.dieSize)
					p = POINT_DICE;
			}
			if (p != POINT_DICE) {
				p = (x-(bs.w/2-bs.barWidth/2 + bs.barWidth)) / bs.checkerSize + 7;
				if (p == 13)
					p = 12;
			}
		}
		if (p > 0) {
			if (y > bs.h/2)
				p = 13 - p;
			else
				p = 12 + p;
			if (board.getDirection() > 0)
				p = 25 - p;
		}
		return(p);
	}
}
