/******************************************************************************
 * BoardTab.java - The board as it appears under the "Board Tab" of the main dialog.
 * $Id: BoardTab.java,v 1.5 2010/12/31 05:30:11 dick Exp $
 * 
 * BuckoFIBS - Backgammon by BuckoSoft
 * Copyright© 2009,2010 - Dick Balaska - BuckoSoft, Corp.
 * 
 * $Log: BoardTab.java,v $
 * Revision 1.5  2010/12/31 05:30:11  dick
 * cvsweb loc has changed.
 *
 * Revision 1.4  2010/12/22 04:37:45  dick
 * matchStatusPanel now needs a reference to the mainDialog.
 *
 * Revision 1.3  2010/03/03 13:12:21  inim
 * Replaced (c) sign in comment mangled by CVS default encoding back to UTF-8
 *
 * Revision 1.2  2010/03/03 12:19:49  inim
 * Moved source to UTF8 encoding from CP1252 encoding. To this end all source files' (c) message was updated to "Copyright© 2009,2010 - Dick Balaska - BuckoSoft, Corp.". This replaces the (c) sign to UTF8, and adds the new year 2010.
 *
 * Revision 1.1  2010/02/04 05:57:53  inim
 * Mavenized project folder layout
 *
 * Revision 1.7  2009/02/14 15:24:21  dick
 * BuckoFIBS is released under the GNU license.
 * Javadoc.
 *
 * Revision 1.6  2009/02/09 22:42:36  dick
 * getBoardGui() is public.
 *
 * Revision 1.5  2009/02/05 06:00:54  dick
 * getGameToolBar needs to be public.
 *
 * Revision 1.4  2009/01/31 08:50:43  dick
 * yourMove(int) takes the number of checkers to move.
 *
 * Revision 1.3  2009/01/31 06:06:10  dick
 * Move the local toolbar foo to GameToolbar.
 *
 * Revision 1.2  2009/01/28 22:10:17  dick
 * Add the ReplayToolbar.
 *
 * Revision 1.1  2009/01/28 19:37:28  dick
 * package com.buckosoft.fibs.gui.board becomes com.buckosoft.fibs.BuckoFIBS.gui.board.
 *
 * Revision 1.4  2009/01/24 17:07:41  dick
 * Add the "game" toolbar.
 *
 * Revision 1.3  2009/01/22 05:05:01  dick
 * yourMove() doesn't need an argument.
 *
 * Revision 1.2  2009/01/18 05:06:56  dick
 * the BoardPanel child becomes a BoardGui.
 *
 * Revision 1.1  2009/01/08 18:31:51  dick
 * The board as it appears under the "Board Tab" of the main dialog.
 */

/* 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * The Original Code is BuckoFIBS, <http://www.buckosoft.com/BuckoFIBS/>.
 * The Initial Developer of the Original Code is Dick Balaska and BuckoSoft, Corp.
 * 
 */
package com.buckosoft.fibs.BuckoFIBS.gui.board;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;

import javax.swing.JPanel;

import com.buckosoft.fibs.BuckoFIBS.CommandDispatcher;
import com.buckosoft.fibs.board.Board;

/** The board as it appears under the "Board Tab" of the main dialog.
 * @author Dick Balaska
 * @since 2009/01/08
 * @version $Revision: 1.5 $ <br> $Date: 2010/12/31 05:30:11 $
 * @see <a href="http://cvs.buckosoft.com/Projects/BuckoFIBS/BuckoFIBS/src/main/java/com/buckosoft/fibs/BuckoFIBS/gui/board/BoardTab.java">cvs BoardTab.java</a>
 */
public class BoardTab extends JPanel {
	private static final long serialVersionUID = 1L;
	private MatchStatusPanel matchStatusPanel = null;
	private BoardGui		boardGui = null;
	private Board			board = new Board();  //  @jve:decl-index=0:
	private GameToolbar		gameToolbar = null;
	private ReplayToolbar	replayToolbar = null;

	/** Create and initialize a BoardTab
	 */
	public BoardTab() {
		super();
		initialize();
	}

	/** Set the reference to our Board
	 * @param board the board to set
	 */
	public void setBoard(Board board) {
		this.board = board;
		if (boardGui != null)
			boardGui.setBoard(board);
		if (this.matchStatusPanel != null)
			this.matchStatusPanel.setBoard(board);
	}

	/** Set the reference to the CommandDispatcher
	 * @param commandDispatcher The application CommandDispatcher
	 */
	public	void	setCommandDispatcher(CommandDispatcher commandDispatcher) {
		this.boardGui.setCommandDispatcher(commandDispatcher);
		this.replayToolbar.setToolbarHandler(commandDispatcher);
		this.gameToolbar.setToolbarHandler(commandDispatcher);
		this.matchStatusPanel.setMainDialog(commandDispatcher.getMainDialog());
	}

	/** Called when the player is to begin his turn.
	 */ 
	public void yourMove(int checkersToMove) {
		this.boardGui.yourMove(checkersToMove);
	}

	/** Repaint the board
	 */
	public void updateBoard() {
		this.boardGui.updateBoard();
		this.boardGui.repaint();
		if (this.matchStatusPanel != null)
			this.matchStatusPanel.updateStatus();
	}
	/**
	 * This method initializes this
	 * 
	 */
	private void initialize() {
        GridBagConstraints gridBagConstraints12 = new GridBagConstraints();
        gridBagConstraints12.fill = GridBagConstraints.VERTICAL;
        gridBagConstraints12.weightx = 1.0;
        gridBagConstraints12.gridx = 1;
        gridBagConstraints12.anchor = GridBagConstraints.EAST;
        gridBagConstraints12.gridy = 0;
        GridBagConstraints gridBagConstraints11 = new GridBagConstraints();
        gridBagConstraints11.fill = GridBagConstraints.VERTICAL;
        gridBagConstraints11.weightx = 1.0;
		gridBagConstraints11.gridx = 0;
		gridBagConstraints11.gridy = 0;
		gridBagConstraints11.anchor = GridBagConstraints.WEST;
        GridBagConstraints gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.anchor = GridBagConstraints.NORTH;
        gridBagConstraints.fill = GridBagConstraints.BOTH;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 0.2;
		gridBagConstraints.gridx = 0;
		gridBagConstraints.gridy = 1;
		gridBagConstraints.gridwidth = 2;
        GridBagConstraints gridBagConstraints1 = new GridBagConstraints();
        gridBagConstraints1.weighty = 10.0;
        gridBagConstraints1.weightx = 1.0;
		gridBagConstraints1.gridx = 0;
		gridBagConstraints1.gridy = 2;
        gridBagConstraints1.fill = GridBagConstraints.BOTH;
        gridBagConstraints1.gridwidth = 2;
        gridBagConstraints1.anchor = GridBagConstraints.SOUTH;
        this.setLayout(new GridBagLayout());
        this.setSize(new Dimension(450, 246));
        this.add(getMatchStatusPanel(), gridBagConstraints);
        this.add(getBoardGui(), gridBagConstraints1);
        this.add(getGameToolBar(), gridBagConstraints11);
        this.add(getJToolBarReplay(), gridBagConstraints12);
			
	}

	/**
	 * This method initializes matchStatusPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getMatchStatusPanel() {
		if (matchStatusPanel == null) {
			matchStatusPanel = new MatchStatusPanel();
			matchStatusPanel.setBoard(board);
		}
		return matchStatusPanel;
	}

	/** Return a handle to the BoardGui
	 * This method initializes boardGui	
	 * @return javax.swing.JPanel	
	 */
	public BoardGui getBoardGui() {
		if (boardGui== null) {
			boardGui = new BoardGui();
			boardGui.setBoard(board);
		}
		return boardGui;
	}

	/**
	 * This method initializes jToolBar	
	 * 	
	 * @return javax.swing.JToolBar	
	 */
	public GameToolbar getGameToolBar() {
		if (gameToolbar == null) {
			gameToolbar = new GameToolbar();
			gameToolbar.setFloatable(false);
		}
		return gameToolbar;
	}

	/**
	 * This method initializes jToolBarReplay	
	 * 	
	 * @return javax.swing.JToolBar	
	 */
	public ReplayToolbar getJToolBarReplay() {
		if (replayToolbar == null) {
			replayToolbar = new ReplayToolbar();
			replayToolbar.setFloatable(false);
		}
		return replayToolbar;
	}

}  //  @jve:decl-index=0:visual-constraint="6,11"
