/******************************************************************************
 * GameToolbar.java - Be the Game Toolbar above the board
 * $Id: GameToolbar.java,v 1.3 2010/03/03 13:12:21 inim Exp $
 * 
 * BuckoFIBS - Backgammon by BuckoSoft
 * Copyright© 2009,2010 - Dick Balaska - BuckoSoft, Corp.
 * 
 * $Log: GameToolbar.java,v $
 * Revision 1.3  2010/03/03 13:12:21  inim
 * Replaced (c) sign in comment mangled by CVS default encoding back to UTF-8
 *
 * Revision 1.2  2010/03/03 12:19:49  inim
 * Moved source to UTF8 encoding from CP1252 encoding. To this end all source files' (c) message was updated to "Copyright© 2009,2010 - Dick Balaska - BuckoSoft, Corp.". This replaces the (c) sign to UTF8, and adds the new year 2010.
 *
 * Revision 1.1  2010/02/04 05:57:53  inim
 * Mavenized project folder layout
 *
 * Revision 1.4  2009/02/14 15:26:52  dick
 * BuckoFIBS is released under the GNU license.
 * Javadoc.
 *
 * Revision 1.3  2009/02/05 05:59:35  dick
 * Add the AskDoubles and Resign buttons.
 *
 * Revision 1.2  2009/01/31 08:51:20  dick
 * Greedy fires Greedy, not Undo.
 *
 * Revision 1.1  2009/01/31 06:03:53  dick
 * Be the Game Toolbar above the board.
 */

/* 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * The Original Code is BuckoFIBS, <http://www.buckosoft.com/BuckoFIBS/>.
 * The Initial Developer of the Original Code is Dick Balaska and BuckoSoft, Corp.
 * 
 */
package com.buckosoft.fibs.BuckoFIBS.gui.board;

import java.awt.Dimension;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JToolBar;

import com.buckosoft.fibs.BuckoFIBS.gui.board.ToolbarHandler.Button;
import java.awt.Rectangle;

/** Be the Game Toolbar above the board
 * @author Dick Balaska
 * @since 2009/01/30
 * @version $Revision: 1.3 $ <br> $Date: 2010/03/03 13:12:21 $
 * @see <a href="http://cvs.buckosoft.com/Projects/BuckoFIBS/BuckoFIBS/src/com/buckosoft/fibs/BuckoFIBS/gui/board/GameToolbar.java">cvs GameToolbar.java</a>
 */
public class GameToolbar extends JToolBar {
	private static final long serialVersionUID = 1L;
	private	ToolbarHandler	toolbarHandler;  //  @jve:decl-index=0:

	private JButton jButtonUndo = null;
	private JButton jButtonGreedy = null;
	private JButton jButtonAskDouble = null;
	private JButton jButtonResign = null;

	private	ImageIcon askDouble = new ImageIcon(getClass().getResource("/g/askDouble.png"));  //  @jve:decl-index=0:
	private	ImageIcon askDoubleOff = new ImageIcon(getClass().getResource("/g/askDoubleOff.png"));

	/** Create and initialize a GameToolbar
	 */
	public GameToolbar() {
		super();
		initialize();
	}

	/** Set the object that will receive our button presses.
	 * @param toolbarHandler the toolbarHandler to set
	 */
	public void setToolbarHandler(ToolbarHandler toolbarHandler) {
		this.toolbarHandler = toolbarHandler;
	}

	/** Set the icon for the AskDouble button to match this state
	 * @param onOff false = draw a line through the icon
	 */ 
	public void setAskDouble(boolean onOff) {
		jButtonAskDouble.setIcon(onOff ? askDouble : askDoubleOff);
	}

	/**
	 * This method initializes this
	 * 
	 */
	private void initialize() {
        this.setBounds(new Rectangle(0, 0, 154, 26));
        this.add(getJButtonUndo());
        this.add(new JToolBar.Separator());
        this.add(getJButtonGreedy());
        this.add(getJButtonAskDouble());
        this.add(new JToolBar.Separator());
        this.add(getJButtonResign());
 			
	}

	/**
	 * This method initializes jButtonUndo	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getJButtonUndo() {
		if (jButtonUndo == null) {
			jButtonUndo = new JButton();
			jButtonUndo.setPreferredSize(new Dimension(18, 18));
			jButtonUndo.setIcon(new ImageIcon(getClass().getResource("/g/undo.png")));
			jButtonUndo.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					toolbarHandler.buttonPressed(Button.Undo);
				}
			});
		}
		return jButtonUndo;
	}

	/**
	 * This method initializes jButtonGreedy	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getJButtonGreedy() {
		if (jButtonGreedy == null) {
			jButtonGreedy = new JButton();
			jButtonGreedy.setPreferredSize(new Dimension(18, 18));
			jButtonGreedy.setIcon(new ImageIcon(getClass().getResource("/g/greedy.png")));
			jButtonGreedy.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					toolbarHandler.buttonPressed(Button.Greedy);
				}
			});
		}
		return jButtonGreedy;
	}

	/**
	 * This method initializes jButtonAutoDoubles	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getJButtonAskDouble() {
		if (jButtonAskDouble == null) {
			jButtonAskDouble = new JButton();
			jButtonAskDouble.setIcon(askDouble);
			jButtonAskDouble.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					toolbarHandler.buttonPressed(Button.AskDoubles);
				}
			});
		}
		return jButtonAskDouble;
	}

	/**
	 * This method initializes jButtonResign	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getJButtonResign() {
		if (jButtonResign == null) {
			jButtonResign = new JButton();
			jButtonResign.setIcon(new ImageIcon(getClass().getResource("/g/whiteflag.png")));
			jButtonResign.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					toolbarHandler.buttonPressed(Button.Resign);
				}
			});
		}
		return jButtonResign;
	}

}  //  @jve:decl-index=0:visual-constraint="10,10"
