/******************************************************************************
 * ChatPane.java - Component that encapsulates the Chat subsystem.
 * $Id: ChatPane.java,v 1.4 2010/12/22 04:38:18 dick Exp $
 * 
 * BuckoFIBS - Backgammon by BuckoSoft
 * Copyright© 2009,2010 - Dick Balaska - BuckoSoft, Corp.
 * 
 * $Log: ChatPane.java,v $
 * Revision 1.4  2010/12/22 04:38:18  dick
 * Select the player that we just maybe added.
 *
 * Revision 1.3  2010/03/03 13:12:22  inim
 * Replaced (c) sign in comment mangled by CVS default encoding back to UTF-8
 *
 * Revision 1.2  2010/03/03 12:19:49  inim
 * Moved source to UTF8 encoding from CP1252 encoding. To this end all source files' (c) message was updated to "Copyright© 2009,2010 - Dick Balaska - BuckoSoft, Corp.". This replaces the (c) sign to UTF8, and adds the new year 2010.
 *
 * Revision 1.1  2010/02/04 05:57:53  inim
 * Mavenized project folder layout
 *
 * Revision 1.4  2010/01/25 21:07:24  dick
 * Allow the program to add players and pre-select which command.
 *
 * Revision 1.3  2009/02/14 15:37:51  dick
 * BuckoFIBS is released under the GNU license.
 *
 * Revision 1.2  2009/02/05 05:58:44  dick
 * Bleh, editing the cvs log to get rid of the toodoo marker.
 *
 * Revision 1.1  2009/01/28 19:38:39  dick
 * package com.buckosoft.fibs.gui.chatWindow becomes com.buckosoft.fibs.BuckoFIBS.gui.chatWindow.
 *
 * Revision 1.7  2009/01/28 08:32:30  dick
 * Get rid of todo tag.
 *
 * Revision 1.6  2009/01/22 05:05:37  dick
 * SEND_CHAT_COMMAND becomes the more generic SEND_COMMAND.
 *
 * Revision 1.5  2009/01/16 07:27:10  dick
 * ChatPane now sends messages.
 *
 * Revision 1.4  2009/01/09 07:19:17  dick
 * Scroll to the bottom on output.
 *
 * Revision 1.3  2008/12/11 08:49:57  dick
 * Inbound chat is being written to the window.
 *
 * Revision 1.2  2008/12/07 22:51:23  dick
 * Javadoc.
 *
 * Revision 1.1  2008/10/08 23:06:04  dick
 * Define the Chat panel.
 */

/* 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * The Original Code is BuckoFIBS, <http://www.buckosoft.com/BuckoFIBS/>.
 * The Initial Developer of the Original Code is Dick Balaska and BuckoSoft, Corp.
 * 
 */
package com.buckosoft.fibs.BuckoFIBS.gui.chatWindow;

import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Rectangle;

import javax.swing.JComboBox;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.JTextPane;
import javax.swing.text.BadLocationException;
import javax.swing.text.StyledDocument;

import com.buckosoft.fibs.BuckoFIBS.CommandDispatcher;


/** Component that encapsulates the Chat subsystem.
 * @author Dick Balaska
 * @since 2008/10/01
 * @version $Revision: 1.4 $ <br> $Date: 2010/12/22 04:38:18 $
 * @see <a href="http://cvs.buckosoft.com/Projects/BuckoFIBS/BuckoFIBS/src/com/buckosoft/fibs/BuckoFIBS/gui/chatWindow/ChatPane.java">cvs ChatPane.java</a>
 */
public class ChatPane extends JPanel {
	private	final static boolean DEBUG = false;
	private static final long serialVersionUID = 1L;

	private	CommandDispatcher	commandDispatcher = null;  //  @jve:decl-index=0:

	private JTextField jChatInput = null;

	private ChatOutput chatOutput = null;
	private	TextPane	textPane;
	private Dimension			chatOutDimension = null;  //  @jve:decl-index=0:
	private	Rectangle			chatOutRectangle = null;

	private JComboBox jCommandComboBox = null;
	private JComboBox jComboBoxPlayer = null;

	private class TextPane extends JTextPane {
		private static final long serialVersionUID = 1L;
	}
	
	/** The possible "chat" command types.
	 */
	public enum Command {
		Command,
		Kibitz,
		Say,
		Shout,
		Tell,
		Whisper
	}

	/**
	 * This is the default constructor
	 */
	public ChatPane() {
		super();
		getChatOutput();
		initialize();
	}

	/** Set the reference to the CommandDispatcher
	 * @param commandDispatcher The application CommandDispatcher
	 */
	public void setCommandDispatcher(CommandDispatcher commandDispatcher) {
		this.commandDispatcher = commandDispatcher;
	}

	/** Write an incoming chat message to the screen
	 * @param s The message
	 */
	public void writeChatMessage(String s) {
		StyledDocument doc = this.textPane.getStyledDocument();
		try {
			doc.insertString(doc.getLength(), s, null);
		} catch (BadLocationException e) {
			System.out.println("Exception ignored...");
			e.printStackTrace();
		}
		chatOutDimension = textPane.getSize(chatOutDimension);
		chatOutRectangle.y = chatOutDimension.height;
		chatOutput.getViewport().scrollRectToVisible(chatOutRectangle);
	}

	private void sendChatCommand() {
		StringBuilder sb = new StringBuilder();
		if (!this.getJComboBoxCommand().getSelectedItem().equals(Command.Command)) {
			sb.append(this.getJComboBoxCommand().getSelectedItem());
			sb.append(" ");
		}
		if (this.getJComboBoxCommand().getSelectedItem().equals(Command.Tell)) {
			String s = (String)this.getJComboBoxPlayer().getSelectedItem();
			maybeAddPlayer(s);
			sb.append(s);
			sb.append(" ");
		}
		sb.append(this.getJChatInput().getText());
		if (DEBUG)
			System.out.println(sb.toString());
		this.commandDispatcher.dispatch(CommandDispatcher.Command.SEND_COMMAND, sb.toString());
		
	}

	/** Maybe add this player to the player dropdown box.
	 * Add it unless it is a duplicate.
	 * @param playerName The name of the player to add to the list
	 */
	public void maybeAddPlayer(String playerName) {
		int z = this.getJComboBoxPlayer().getItemCount();
		boolean found = false;
		for (int i=0; i<z; i++) {
			if (this.getJComboBoxPlayer().getItemAt(i).equals(playerName)) {
				found = true;
				break;
			}
		}
		if (!found)
			this.getJComboBoxPlayer().addItem(playerName);
		this.getJComboBoxPlayer().setSelectedItem(playerName);
	}

	/** This method initializes this
	 * @return void
	 */
	private void initialize() {
		GridBagConstraints gridBagConstraints11 = new GridBagConstraints();
		gridBagConstraints11.fill = GridBagConstraints.VERTICAL;
		gridBagConstraints11.gridy = 1;
		gridBagConstraints11.weightx = 1.0;
		gridBagConstraints11.anchor = GridBagConstraints.WEST;
		gridBagConstraints11.gridx = 1;
		GridBagConstraints gridBagConstraints1 = new GridBagConstraints();
		gridBagConstraints1.fill = GridBagConstraints.NONE;
		gridBagConstraints1.gridy = 1;
		gridBagConstraints1.weightx = 1.0;
		gridBagConstraints1.anchor = GridBagConstraints.WEST;
		gridBagConstraints1.gridx = 0;
		GridBagConstraints gridBagConstraints4 = new GridBagConstraints();
		gridBagConstraints4.fill = GridBagConstraints.BOTH;
		gridBagConstraints4.gridx = 2;
		gridBagConstraints4.gridy = 1;
		gridBagConstraints4.ipadx = 311;
		gridBagConstraints4.ipady = 0;
		gridBagConstraints4.weightx = 100.0;
		gridBagConstraints4.insets = new Insets(0, 0, 0, 0);
		GridBagConstraints gridBagConstraints3 = new GridBagConstraints();
		gridBagConstraints3.gridy = 0;
		gridBagConstraints3.weightx = 2.0D;
		gridBagConstraints3.weighty = 2.0D;
		gridBagConstraints3.anchor = GridBagConstraints.NORTH;
		gridBagConstraints3.fill = GridBagConstraints.BOTH;
		gridBagConstraints3.gridwidth = 3;
		gridBagConstraints3.gridx = 0;
		GridBagConstraints gridBagConstraints = new GridBagConstraints();
		gridBagConstraints.gridx = 0;
		gridBagConstraints.gridy = 0;
		this.setLayout(new GridBagLayout());
		this.setSize(315, 221);
		this.add(getChatOutput(), gridBagConstraints3);
		this.add(getJChatInput(), gridBagConstraints4);
		this.add(getJComboBoxCommand(), gridBagConstraints1);
		this.add(getJComboBoxPlayer(), gridBagConstraints11);
		setupComboBoxPlayer();
	}

	private void setupComboBoxPlayer() {
		if (getJComboBoxCommand().getSelectedItem().equals(Command.Tell))
			getJComboBoxPlayer().setVisible(true);
		else
			getJComboBoxPlayer().setVisible(false);
	}

	/** Make the player combobox point to this type
	 * @param type The command type to set the combo box to.
	 */
	public void setupComboBoxCommand(Command type) {
		getJComboBoxCommand().setSelectedItem(type);
	}
	
	/**
	 * This method initializes jChatInput	
	 * 	
	 * @return javax.swing.JTextField	
	 */
	private JTextField getJChatInput() {
		if (jChatInput == null) {
			jChatInput = new JTextField();
			jChatInput.setPreferredSize(new Dimension(315, 25));
			jChatInput.setFont(new Font("Dialog", Font.PLAIN, 12));
			jChatInput.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					sendChatCommand();
				}
			});
		}
		return jChatInput;
	}

	/**
	 * This method initializes chatOutput	
	 * 	
	 * @return javax.swing.JScrollPane	
	 */
	private ChatOutput getChatOutput() {
		if (chatOutput == null) {
			textPane = new TextPane();
			chatOutput = new ChatOutput(textPane);
			chatOutput.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
			chatOutDimension = new Dimension(10,10);
			chatOutRectangle = new Rectangle(10,10);
		}
		return chatOutput;
	}

	/**
	 * This method initializes jCommandComboBox	
	 * 	
	 * @return javax.swing.JComboBox	
	 */
	private JComboBox getJComboBoxCommand() {
		if (jCommandComboBox == null) {
			jCommandComboBox = new JComboBox();
			jCommandComboBox.setPreferredSize(new Dimension(100, 25));
			jCommandComboBox.addItemListener(new java.awt.event.ItemListener() {
				public void itemStateChanged(java.awt.event.ItemEvent e) {
					if (DEBUG)
						System.out.println("itemStateChanged()");
					setupComboBoxPlayer();
				}
			});
			for (Command c : Command.values())
				jCommandComboBox.addItem(c);
		}
		return jCommandComboBox;
	}

	/**
	 * This method initializes jComboBoxPlayer	
	 * 	
	 * @return javax.swing.JComboBox	
	 */
	private JComboBox getJComboBoxPlayer() {
		if (jComboBoxPlayer == null) {
			jComboBoxPlayer = new JComboBox();
			jComboBoxPlayer.setEditable(true);
			jComboBoxPlayer.setPreferredSize(new Dimension(100, 25));
		}
		return jComboBoxPlayer;
	}
}  //  @jve:decl-index=0:visual-constraint="10,10"
