/******************************************************************************
 * EditGroupDialog.java - Allow the user to edit a GroupOfPlayers
 * $Id: EditGroupDialog.java,v 1.1 2011/01/01 20:31:06 dick Exp $
 * 
 * BuckoFIBS - Backgammon by BuckoSoft
 * Copyright© 2010 - Dick Balaska - BuckoSoft, Corp.
 * 
 * $Log: EditGroupDialog.java,v $
 * Revision 1.1  2011/01/01 20:31:06  dick
 * NewGroupDialog becomes more generic EditGroupDialog.
 *
 * Revision 1.1  2010/12/31 05:45:42  dick
 * Allow the user to define a new GroupOfPlayers.
 */

/* 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * The Original Code is BuckoFIBS, <http://www.buckosoft.com/BuckoFIBS/>.
 * The Initial Developer of the Original Code is Dick Balaska and BuckoSoft, Corp.
 * 
 */
package com.buckosoft.fibs.BuckoFIBS.gui.playerList.group;

import java.awt.Color;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;

import com.buckosoft.fibs.domain.GroupOfPlayers;

/** Allow the user to edit a GroupOfPlayers
 * @author Dick Balaska
 * @since 2010/12/30
 * @version $Revision: 1.1 $ <br> $Date: 2011/01/01 20:31:06 $
 * @see <a href="http://cvs.buckosoft.com/Projects/BuckoFIBS/BuckoFIBS/src/main/java/com/buckosoft/fibs/BuckoFIBS/gui/playerList/group/EditGroupDialog.java">cvs EditGroupDialog.java</a>
 */
public class EditGroupDialog extends JDialog {

	private static final long serialVersionUID = 1L;
	private	GroupSelectPane groupSelectPane;

	private JPanel jContentPane = null;
	private JLabel jLabel = null;
	private JTextField jTextFieldName = null;
	private JLabel jLabel1 = null;
	private JTextField jTextFieldColor = null;
	private JLabel jLabel2 = null;
	private JCheckBox jCheckBoxIgnore = null;
	private JLabel jLabel3 = null;
	private JCheckBox jCheckBoxActive = null;
	private JPanel jPanelButtons = null;
	private JButton jButtonOK = null;
	private JButton jButtonCancel = null;

	private	GroupOfPlayers	groupOfPlayers = null;

	/**
	 * @param owner
	 */
	public EditGroupDialog(JFrame owner, GroupSelectPane groupSelectPane) {
		super(owner, true);
		this.groupSelectPane = groupSelectPane;
		initialize();
		this.setLocationRelativeTo(owner);
	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
		this.setSize(300, 200);
		this.setTitle("Define New Group");
		this.setContentPane(getJContentPane());
	}


	/** If we are editing an existing group, set it here.
	 * Otherwise, we are creating a new group.
	 * @param groupOfPlayers The GroupOfPlayers to edit.
	 */
	public void setGroupOfPlayers(GroupOfPlayers groupOfPlayers) {
		this.groupOfPlayers = groupOfPlayers;
		this.setTitle("Edit Group");
		this.jTextFieldName.setText(groupOfPlayers.getGroupName());
		this.jCheckBoxIgnore.getModel().setSelected(groupOfPlayers.isIgnore());
		this.jCheckBoxActive.getModel().setSelected(groupOfPlayers.isActive());
		
	}

	/**
	 * This method initializes jContentPane
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJContentPane() {
		if (jContentPane == null) {
			GridBagConstraints gridBagConstraints8 = new GridBagConstraints();
			gridBagConstraints8.gridx = 0;
			gridBagConstraints8.gridwidth = 2;
			gridBagConstraints8.anchor = GridBagConstraints.SOUTH;
			gridBagConstraints8.weighty = 0.2;
			gridBagConstraints8.insets = new Insets(0, 0, 5, 0);
			gridBagConstraints8.gridy = 4;
			GridBagConstraints gridBagConstraints7 = new GridBagConstraints();
			gridBagConstraints7.gridx = 1;
			gridBagConstraints7.anchor = GridBagConstraints.WEST;
			gridBagConstraints7.fill = GridBagConstraints.HORIZONTAL;
			gridBagConstraints7.gridy = 3;
			GridBagConstraints gridBagConstraints6 = new GridBagConstraints();
			gridBagConstraints6.gridx = 0;
			gridBagConstraints6.gridy = 3;
			jLabel3 = new JLabel();
			jLabel3.setText("Active?");
			GridBagConstraints gridBagConstraints5 = new GridBagConstraints();
			gridBagConstraints5.gridx = 1;
			gridBagConstraints5.anchor = GridBagConstraints.WEST;
			gridBagConstraints5.fill = GridBagConstraints.HORIZONTAL;
			gridBagConstraints5.gridy = 2;
			GridBagConstraints gridBagConstraints4 = new GridBagConstraints();
			gridBagConstraints4.gridx = 0;
			gridBagConstraints4.gridy = 2;
			jLabel2 = new JLabel();
			jLabel2.setText("Ignore?");
			GridBagConstraints gridBagConstraints3 = new GridBagConstraints();
			gridBagConstraints3.fill = GridBagConstraints.BOTH;
			gridBagConstraints3.gridy = 1;
			gridBagConstraints3.weightx = 1.0;
			gridBagConstraints3.gridx = 1;
			GridBagConstraints gridBagConstraints2 = new GridBagConstraints();
			gridBagConstraints2.gridx = 0;
			gridBagConstraints2.gridy = 1;
			jLabel1 = new JLabel();
			jLabel1.setText("Color:");
			GridBagConstraints gridBagConstraints1 = new GridBagConstraints();
			gridBagConstraints1.fill = GridBagConstraints.BOTH;
			gridBagConstraints1.gridy = 0;
			gridBagConstraints1.weightx = 1.0;
			gridBagConstraints1.gridx = 1;
			GridBagConstraints gridBagConstraints = new GridBagConstraints();
			gridBagConstraints.gridx = 0;
			gridBagConstraints.gridy = 0;
			jLabel = new JLabel();
			jLabel.setText("Name:");
			jContentPane = new JPanel();
			jContentPane.setLayout(new GridBagLayout());
			jContentPane.add(jLabel, gridBagConstraints);
			jContentPane.add(getJTextFieldName(), gridBagConstraints1);
			jContentPane.add(jLabel1, gridBagConstraints2);
			jContentPane.add(getJTextFieldColor(), gridBagConstraints3);
			jContentPane.add(jLabel2, gridBagConstraints4);
			jContentPane.add(getJCheckBoxIgnore(), gridBagConstraints5);
			jContentPane.add(jLabel3, gridBagConstraints6);
			jContentPane.add(getJCheckBoxActive(), gridBagConstraints7);
			jContentPane.add(getJPanelButtons(), gridBagConstraints8);
		}
		return jContentPane;
	}

	/**
	 * This method initializes jTextFieldName	
	 * 	
	 * @return javax.swing.JTextField	
	 */
	private JTextField getJTextFieldName() {
		if (jTextFieldName == null) {
			jTextFieldName = new JTextField();
		}
		return jTextFieldName;
	}

	/**
	 * This method initializes jTextFieldColor	
	 * 	
	 * @return javax.swing.JTextField	
	 */
	private JTextField getJTextFieldColor() {
		if (jTextFieldColor == null) {
			jTextFieldColor = new JTextField();
			jTextFieldColor.setEditable(false);
			jTextFieldColor.setText("Black");
		}
		return jTextFieldColor;
	}

	/**
	 * This method initializes jCheckBoxIgnore	
	 * 	
	 * @return javax.swing.JCheckBox	
	 */
	private JCheckBox getJCheckBoxIgnore() {
		if (jCheckBoxIgnore == null) {
			jCheckBoxIgnore = new JCheckBox();
		}
		return jCheckBoxIgnore;
	}

	/**
	 * This method initializes jCheckBoxActive	
	 * 	
	 * @return javax.swing.JCheckBox	
	 */
	private JCheckBox getJCheckBoxActive() {
		if (jCheckBoxActive == null) {
			jCheckBoxActive = new JCheckBox();
		}
		return jCheckBoxActive;
	}

	/**
	 * This method initializes jPanelButtons	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getJPanelButtons() {
		if (jPanelButtons == null) {
			GridBagConstraints gridBagConstraints10 = new GridBagConstraints();
			gridBagConstraints10.insets = new Insets(0, 0, 0, 10);
			gridBagConstraints10.anchor = GridBagConstraints.SOUTH;
			GridBagConstraints gridBagConstraints9 = new GridBagConstraints();
			gridBagConstraints9.insets = new Insets(0, 10, 0, 0);
			gridBagConstraints9.anchor = GridBagConstraints.SOUTH;
			jPanelButtons = new JPanel();
			jPanelButtons.setLayout(new GridBagLayout());
			jPanelButtons.add(getJButtonOK(), gridBagConstraints10);
			jPanelButtons.add(getJButtonCancel(), gridBagConstraints9);
		}
		return jPanelButtons;
	}

	/**
	 * This method initializes jButtonOK	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getJButtonOK() {
		if (jButtonOK == null) {
			jButtonOK = new JButton();
			jButtonOK.setText("OK");
			jButtonOK.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					onOK();
				}
			});
		}
		return jButtonOK;
	}

	/**
	 * This method initializes jButtonCancel	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getJButtonCancel() {
		if (jButtonCancel == null) {
			jButtonCancel = new JButton();
			jButtonCancel.setText("Cancel");
			jButtonCancel.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					onCancel();
				}
			});
		}
		return jButtonCancel;
	}

	private void onOK() {
		GroupOfPlayers gop;
		if (this.groupOfPlayers != null)
			gop = this.groupOfPlayers;
		else
			gop = new GroupOfPlayers();
		gop.setGroupName(jTextFieldName.getText());
		if (gop.getGroupName() == null || gop.getGroupName().length() == 0) {
			JOptionPane.showMessageDialog(this,
				    "Group name must not be empty.",
				    "Error",
				    JOptionPane.ERROR_MESSAGE);
			return;
		}
		gop.setActive(this.jCheckBoxActive.getModel().isSelected());
		gop.setIgnore(this.jCheckBoxIgnore.getModel().isSelected());
		gop.setColor(Color.black);
		if (this.groupOfPlayers != null)
			this.groupSelectPane.editGroupFromUser(gop);
		else
			this.groupSelectPane.addNewGroupFromUser(gop);
		this.setVisible(false);
	}
	
	private void onCancel() {
		this.setVisible(false);
	}
}
