/******************************************************************************
 * GroupPopupSubmenu.java - The group submenu for putting a player in groups
 * $Id: GroupPopupSubmenu.java,v 1.3 2011/01/01 20:43:12 dick Exp $
 * 
 * BuckoFIBS - Backgammon by BuckoSoft
 * Copyright© 2010 - Dick Balaska - BuckoSoft, Corp.
 * 
 * $Log: GroupPopupSubmenu.java,v $
 * Revision 1.3  2011/01/01 20:43:12  dick
 * Turn off debug.
 *
 * Revision 1.2  2011/01/01 06:22:09  dick
 * Javadoc.
 *
 * Revision 1.1  2011/01/01 00:17:56  dick
 * The group submenu for putting a player in groups
 *
 */

/* 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * The Original Code is BuckoFIBS, <http://www.buckosoft.com/BuckoFIBS/>.
 * The Initial Developer of the Original Code is Dick Balaska and BuckoSoft, Corp.
 * 
 */
package com.buckosoft.fibs.BuckoFIBS.gui.playerList.group;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.LinkedList;

import javax.swing.JCheckBoxMenuItem;
import javax.swing.JMenu;

import com.buckosoft.fibs.BuckoFIBS.GroupManager;
import com.buckosoft.fibs.domain.Player;

/** The group submenu for putting a player in groups
 * @author Dick Balaska
 * @since 2010/12/31
 * @version $Revision: 1.3 $ <br> $Date: 2011/01/01 20:43:12 $
 * @see <a href="http://cvs.buckosoft.com/Projects/BuckoFIBS/BuckoFIBS/src/main/java/com/buckosoft/fibs/BuckoFIBS/gui/playerList/group/GroupPopupSubmenu.java">cvs GroupPopupSubmenu.java</a>
 */
public class GroupPopupSubmenu extends JMenu implements ActionListener {
	private	final static boolean DEBUG = false;
	private static final long serialVersionUID = 1L;

	private GroupManager groupManager;
	
	
	/** A small helper class that contains a group name and its selected status.
	 * Used for building the GroupPopupSubmenu
	 * @author Dick Balaska
	 * @since 2010/12/31
	 * @version $Revision: 1.3 $ <br> $Date: 2011/01/01 20:43:12 $
	 * @see <a href="http://cvs.buckosoft.com/Projects/BuckoFIBS/BuckoFIBS/src/main/java/com/buckosoft/fibs/BuckoFIBS/gui/playerList/group/GroupPopupSubmenu.java">cvs GroupPopupSubmenu.java</a>
	 */
	public class PopupGroup {
		/** The name of the group to display */
		public String name;

		/** Draw this entry as selected */
		public boolean selected;
	}

	/** Default constructor and initialize. 
	 */
	public GroupPopupSubmenu() {
		super();
		initialize();
	}

	/** Called when a menu entry from this menu is selected.
	 * @param e What just happened?
	 */
	public void actionPerformed(ActionEvent e) {
		//this.groupManager.menuEvent(e);
		String s = e.getActionCommand();
		int i = s.indexOf('-');
		String gn = s.substring(0, i);
		String pn = s.substring(i+1);
		if (DEBUG)
			System.out.println("menu: '" + s + "' gn='" + gn + "' pn='" + pn + "'");
		this.groupManager.popupMenuItemSelected(gn, pn);
	}

	/**
	 * This method initializes this
	 * 
	 */
	private void initialize() {
        this.setSize(new Dimension(73, 40));
        this.setText("Groups");
	}

	/**
	 * @return the gameManager
	 */
	public GroupManager getGroupManager() {
		return groupManager;
	}

	/** Set our reference to the object that will receive menu events
	 * @param groupManager the groupManager to set
	 */
	public void setGroupManager(GroupManager groupManager) {
		this.groupManager = groupManager;
	}
	
	/** Empty and recreate the entries in the popup menu for this player.
	 * @param player The player who we right-clicked on.
	 */
	public void repopulate(Player player) {
		if (DEBUG)
			System.out.println("GroupPopupSubmenu: repopulate for player " + player.getId() + " '" + player.getName() + "'");
		this.removeAll();
		LinkedList<PopupGroup> lpg = this.groupManager.getGroupsForPlayer(this, player.getId());
		for (PopupGroup pg : lpg) {
			if (pg.name.equals("All"))
				continue;
			JCheckBoxMenuItem mi  = new JCheckBoxMenuItem();
			mi.setText(pg.name);
			mi.setSelected(pg.selected);
			//mi.setSelected(true);
			mi.setActionCommand(pg.name + "-" + player.getName());
			mi.setToolTipText("Add this player to this group.");
			mi.addActionListener(this);
			this.add(mi);
		}
	}
}
