/******************************************************************************
 * WinLoss - Describe the Win Loss record against one player.
 * $Id: WinLoss.java,v 1.2 2010/12/30 17:38:19 dick Exp $
 * 
 * BuckoFIBS - Backgammon by BuckoSoft
 * Copyright 2010 - Dick Balaska - BuckoSoft, Corp.
 * 
 * $Log: WinLoss.java,v $
 * Revision 1.2  2010/12/30 17:38:19  dick
 * Correct the cvs link.
 *
 * Revision 1.1  2010/12/24 02:58:03  dick
 * Describe the Win Loss record against one player.
 *
 */

/* 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * The Original Code is BuckoFIBS, <http://www.buckosoft.com/BuckoFIBS/>.
 * The Initial Developer of the Original Code is Dick Balaska and BuckoSoft, Corp.
 * 
 */
package com.buckosoft.fibs.domain;


/** A class to manage wins/losses for a Player.
 * We need to be able to deal with them as numbers and present them as a string.
 * @author dick
 * @since 2010/12/23
 * @version $Revision: 1.2 $ <br> $Date: 2010/12/30 17:38:19 $
 * @see <a href="http://cvs.buckosoft.com/Projects/BuckoFIBS/BuckoFIBS/src/main/java/com/buckosoft/fibs/domain/WinLoss.java">cvs WinLoss.java</a>
 *
 */
public class WinLoss  implements Comparable<WinLoss> {
	private int		wins;
	private int		losses;

	/** Convienence constructor that takes the wins and losses
	 * @param wins The number of times you have beaten this opponent.
	 * @param losses The number of times you have lost to this opponent.
	 */
	public WinLoss(int wins, int losses) {
		this.wins = wins;
		this.losses = losses;
	}

	public WinLoss() {
	}

	/** Get the numeric value of this WinLoss
	 * @return wins/losses, more or less
	 */
	public double	getValue() {
		if (losses == 0)
			return(wins*10);
		if (wins == 0)
			return(-losses);
		return((double)wins/(double)losses);
	}
	
	/** Get the String value of this WinLoss
	 * @return "W-L"
	 */
	public String toString() {
		return("" + wins + "-" + losses);
	}

	/** String compare the name of this player with that one.
	 * @param o The other player to compare
	 */
	@Override
	public int compareTo(WinLoss o) {
		if (this.getValue() < o.getValue())
			return(-1);
		if (this.getValue() > o.getValue())
			return(1);
		return(0);
	}
}

